//###################################################################################
//                                                                            
// LCDHype Crystal Fontz LCD Driver DLL Source Code                           
//                                                                            
// Version 1.01b                                                                
//                                                                            
// Requirements: LCDHype Version 0.37 or above                                
//                                                                            
//                                                                            
// hk@holger-klabunde.de                                                      
// http://www.holger-klabunde.de                                              
// 04.01.2006
// This is FREE software
//###################################################################################

#include "stdafx.h"
#include "lcdllc.h"
#include "windows.h"
#include <stdio.h>
#include <direct.h>

#include "serial.h"

#define CURSOR_HOME		1
#define CLEAR_SCREEN	12
#define CURSOR_XY		17
#define CG_CHAR			25
#define SET_LIGHT		14
#define SET_CONTRAST	15

            //LCD Driver DLL Identifier String
const char* ID            = "Serial Crystal Fontz Driver V1.01b";

            //maximum Pixels of graphics mode, 76800 = 320x240
const int   MAX_RESPIXELS = 76800;
      
//Custom Char Data Structure
struct TCharData 
{ BYTE position;   //Position of custom char in ram
  BYTE data[8][8]; //custom char 8x8 pixel data matrix
};

//LCD Driver Information Structure
struct TDLLInfo
{
  char IDArray[256];          //holds identifier of DLL
  bool SupportGfxLCD;         //does this driver support graphical LCDs?
  bool SupportTxtLCD;         //does this driver support text
  bool SupportLightSlider;    //does this driver support the light control slider
  bool SupportContrastSlider; //does this driver support the contrast slider
  bool SupportOutports;		  //does this driver support outports for controlling external circuits
  BYTE CCharWidth;            //custom char width in pixels
  BYTE CCharHeight;           //custom char height in pixels
  BYTE FontPitch;             //fontpitch of LCD in pixels
};

//Structure for Symbol LCDs
struct TSymbolInfo
{ 
  bool SymbolData[20]; //is symbol x displayed or not?
};

//global storage for COM port actions
HANDLE porth=0; //Handle to COM Port
int baudrate=9600;
char line[256];		//only error messages 

//global storage for LCD propertys
bool  LCD_LightOn     = true;  //is lcd light on?
char LCD_Port[32]     = "COM1"; //port where lcd is connected to
DWORD LCD_Exectime    = 10;    //execution time of lcd in textmode
DWORD LCD_ExectimeGfx = 10;    //execution time of lcd in graphics mode
DWORD LCD_Width       = 240;   //width of LCDs graphics mode in pixels
DWORD LCD_Height      = 64;   //height of LCDs graphics mode in pixels 
DWORD LCD_Cols        = 40;    //number of chars per line in text mode
DWORD LCD_Rows        = 8;    //number of LCD lines in text mode
LONG  LCD_FontPitch    = 6;    //fontpitch of LCD in pixels   
BYTE  LCD_ContrastValue = 0;    
BYTE  LCD_LightValue    = 0;

char ininame[_MAX_PATH]=""; //Die INI-Datei
//char drive[_MAX_DRIVE]=""; //Hilfsarrays
char dir[_MAX_DIR]="";
//char name[_MAX_FNAME]="";
//char ext[_MAX_EXT]="";

//####################################################################################################
int ReadConfigFile()
//####################################################################################################
{

//if not found config.ini, use default settings

 baudrate=GetPrivateProfileInt("CrystalFontz","BaudRate",19200,ininame);
 textbuffersize=GetPrivateProfileInt("CrystalFontz","TextBufferSize",1,ininame);
 if(textbuffersize<=0 || textbuffersize>BUFFER_SIZE) textbuffersize=1;

//  sprintf(line,"##################### '%s' ! #####################\nBaudrate %d",ininame,baudrate);
//  MessageBox(NULL,line,NULL,MB_OK);
 return 0;
}


//####################################################################################################
int WriteConfigFile()
//####################################################################################################
{
 _itoa(baudrate,line,10);
 WritePrivateProfileString("CrystalFontz","BaudRate",line,ininame);
 _itoa(textbuffersize,line,10);
 WritePrivateProfileString("CrystalFontz","TextBufferSize",line,ininame);

/*
 _itoa(LCD_ContrastValue,line,10);
 WritePrivateProfileString("CrystalFontz","ContrastValue",line,ininame);
 _itoa(LCD_LightValue,line,10);
 WritePrivateProfileString("CrystalFontz","LightValue",line,ininame);
*/

 return 0;
}


//####################################################################################################
//function used to send data to lpt port based on lcd controller
void Send(ULONG Port, 
          BYTE High,
          BYTE Low,
          BYTE Value,
          DWORD Exectime)
//####################################################################################################
{ 
//  DWORD i;

//  DlPortWritePortUchar(LCD_Port+2,High+((BYTE)(!LCD_LightOn))*Light_Byte);
//  DlPortWritePortUchar(LCD_Port,Value);
//  for (i=0;i<=Exectime;i++) { DlPortReadPortUchar(0x220); };
//  DlPortWritePortUchar(LCD_Port+2,Low+((BYTE)(!LCD_LightOn))*Light_Byte);
//  for (i=0;i<=Exectime;i++) { DlPortReadPortUchar(0x220); };
}

//function used to create a lcd driver config dialog
//dont export this function if you dont want to create config dialog
//####################################################################################################
void _stdcall LCD_ConfigDialog()
//####################################################################################################
{
 sprintf(line,"Baudrate %d\nBacklight %d\nContrast %d\nTextBufferSize %d\n"
	     ,baudrate
		 ,(int)LCD_LightValue
		 ,(int)LCD_ContrastValue
		 ,textbuffersize);

 MessageBox(NULL,line,"Settings",MB_OK);

// WriteConfigFile();
};

//function used to send a byte to lcd controller
//####################################################################################################
void _stdcall LCD_SendToController(BYTE Value)
//####################################################################################################
{ };

//####################################################################################################
//function used to send a byte to lcd memory
//or better write a byte on the display
void _stdcall LCD_SendToMemory(BYTE Value)
//####################################################################################################
{
 SerialPutToBuffer(&porth,Value);
};

//####################################################################################################
//function used to send gfx data to graphical lcd
void _stdcall LCD_SendToGfxMemory(BYTE Pixels[MAX_RESPIXELS], //pixel color data
                                  WORD x1,                    //refresh area x1,y1,x2,y2
                                  WORD y1, 
                                  WORD x2,
                                  WORD y2,
                                  bool inverted               //output of gfx data inverted?
                                 )
//####################################################################################################
{ };

//####################################################################################################
//function to set the output address in text mode
//set the cursor to position x (coloumn),y (row)
//x and y are zero based positions !
void _stdcall LCD_SetOutputAddress(DWORD x, DWORD y)
//####################################################################################################
{
 SerialPutToBuffer(&porth,CURSOR_XY);
 SerialPutToBuffer(&porth,(unsigned char)x);
 SerialPutToBuffer(&porth,(unsigned char)y);

// SerialFlushBuffer(&porth);
};

//####################################################################################################
//function to write a custom char to ram
//each custom char comes in an 8x8 PIXEL matrix !
void _stdcall LCD_SetCGRAMChar(TCharData cdata)
//####################################################################################################
{
 unsigned char by,x,y,mask;

 SerialPutToBuffer(&porth,CG_CHAR);
 SerialPutToBuffer(&porth,(unsigned char)cdata.position);

 for(x=0; x<8; x++) // x = rows of the custom char
 {

  by=0;
 
  mask=0x10; //MSB first 

  for(y=0; y<5; y++) // y = col of the custom char
  {
   if(cdata.data[y][x] > 0) by|=mask;

   mask>>=1;
  }

  SerialPutToBuffer(&porth,by);

 }

// SerialFlushBuffer(&porth);
};

//####################################################################################################
//function to get the position of custom char
//IN : custom char position based on HD44780 LCDs (0...)
//OUT: custom char position converted to current lcd controllers position
//
//Example : HD44780, first custom char is at position 0
//          T6963c , first custom char is at position 128
//
//          function LCD_GetCGRAMChar gets 1 and returns 128
BYTE _stdcall LCD_GetCGRAMChar(BYTE position)
//####################################################################################################
{
 return position+128; //return char code offset for Custom Characters with Crystal Fontz Display
}; 


//####################################################################################################
//function used to clean up driver usage, called when DLL is closed.
void _stdcall LCD_CleanUp()
//####################################################################################################
{
 SerialFlushBuffer(&porth);

 //Clear Screen Command
 SerialPutc(&porth,CLEAR_SCREEN);

 //Contrast Off 
 SerialPutc(&porth,SET_CONTRAST);
 SerialPutc(&porth,0);

//Backlight off
 SerialPutc(&porth,SET_LIGHT);
 SerialPutc(&porth,0);

 Sleep(100);

 if(porth!=0) //clean up the COM port
 {
	EscapeCommFunction(porth, CLRDTR);
	EscapeCommFunction(porth, CLRRTS);
    CloseHandle(porth);
    porth=0;
 }

};

//####################################################################################################
//function used to initialize LCD
void _stdcall LCD_Init(void)
//####################################################################################################
{

  _getcwd(dir,_MAX_DIR); //get current working directory
  _makepath(ininame,NULL,dir,"config","ini");

  ReadConfigFile();
  WriteConfigFile(); //make default config.ini if file does not exist

//lcdhype calls this function more than one time !
//if porth is a valid handle, than CLOSE it first,
//or WIN will report "Access denied" for the already
//open COM-Port !
 if(porth!=0) LCD_CleanUp();

 if(porth==0)
 {
  porth=SerialInit(LCD_Port, baudrate); 
  if(porth==0 && porth!=INVALID_HANDLE_VALUE)
  {
//  sprintf(line,"##################### '%s' not available ! #####################",LCD_Port);
//  MessageBox(NULL,line,NULL,MB_OK);
   return;
  }
 }

 SerialPutc(&porth,CLEAR_SCREEN);
 Sleep(100);
};

//####################################################################################################
//function used to indicate that the lcd can receive data
//commonly used when interfacing serial lcds else return value true
bool _stdcall LCD_IsReadyToReceive()
//####################################################################################################
{ return true; }; 



//####################################################################################################
//function used to set the lcd propertys, so that the lcd driver can work correctly
//commonly used when supporting more than one lcd type in one driver
void _stdcall LCD_SetIOPropertys(char* Port,                //port lcd is connected to
								 DWORD Exectime,            //execution time for text mode
								 DWORD ExectimeGfx,         //execution time for gfx mode
								 DWORD X,                   //cols in text mode
								 DWORD Y,                   //rows in text mode
								 DWORD GX,                  //width in pixels in gfx mode
								 DWORD GY,                  //height in pixels in gfx mode
								 bool  LightOn,             //is light control enabled?
								 BYTE  LightSliderValue,    //brightness value from light slider
								 bool  ContrastOn,          //is contrast control enabled?
								 BYTE  ContrastSliderValue, //contrast value from contrast slider
								 DWORD Outports,            //32bit = 32 outports, bit=0=off, bit=1=on 
								 bool  UnderlineMode,       //is underline mode in text mode enabled?
								 bool  UnderlineOutput      //display underlined text?
								)
//####################################################################################################
{ 
 //This function is called periodically by lcdhype !

 strcpy(LCD_Port,Port);
 LCD_Exectime=Exectime;
 LCD_Cols=X;
 LCD_Rows=Y;

 LCD_LightValue    = LightSliderValue/2;
 LCD_ContrastValue = ContrastSliderValue/2;    


 if(LCD_LightValue>100) LCD_LightValue=100;
 if(LCD_ContrastValue>100) LCD_ContrastValue=100;

 if(LightOn)
 {
  SerialPutToBuffer(&porth,SET_LIGHT);
  SerialPutToBuffer(&porth,LCD_LightValue);   //Backlight on
//  SerialFlushBuffer(&porth);
 }
 else
 {
  SerialPutToBuffer(&porth,SET_LIGHT);
  SerialPutToBuffer(&porth,0); //Backlight off
//  SerialFlushBuffer(&porth);
 }

 SerialPutToBuffer(&porth,SET_CONTRAST);
 SerialPutToBuffer(&porth,LCD_ContrastValue);
// SerialFlushBuffer(&porth);
};

//function used to interface symbols of a symbol lcd
void _stdcall LCD_Customize(TSymbolInfo SymData)
{ };

//####################################################################################################
//function used to return information about the dll to the calling process
TDLLInfo _stdcall DLL_GetInfo()
//####################################################################################################
{ 
  TDLLInfo result;  
  result.CCharWidth  = 6;       
  result.CCharHeight = 8;
  result.FontPitch = 6;
  result.SupportGfxLCD = false;
  result.SupportTxtLCD = true;
  result.SupportLightSlider = true;
  result.SupportContrastSlider = true;
  result.SupportOutports = false;

  ZeroMemory(&result.IDArray,256);
  CopyMemory(&result.IDArray[0],&ID[0],strlen(ID));

  return result;
};

